@@ -1,19 +1,19 @@
 import numpy as np
 
 from glotaran.analysis.simulation import simulate
+from glotaran.analysis.test.models import SimpleTestModel
 from glotaran.parameter import ParameterGroup
 
-from .models import SimpleTestModel
-
 
 def test_simulate_dataset():
     model = SimpleTestModel.from_dict(
         {
+            "megacomplex": {"m1": {"grouped": False, "indexdependent": False}},
             "dataset": {
                 "dataset1": {
-                    "megacomplex": [],
+                    "megacomplex": ["m1"],
                 },
-            }
+            },
         }
     )
     print(model.validate())
