@@ -16,10 +16,11 @@ if TYPE_CHECKING:
     from typing import Any
     from typing import Sequence
 
+    from glotaran.builtin.models.kinetic_spectrum.kinetic_spectrum_model import (
+        KineticSpectrumModel,
+    )
     from glotaran.parameter import ParameterGroup
 
-    from .kinetic_spectrum_model import KineticSpectrumModel
-
 
 @model_attribute(
     properties={
